﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;
using System.Drawing;
using Microsoft.Win32;

namespace Dock
{
    #region Flags

    [Flags]
    public enum SHGFI : int
    {
        /// <summary>get icon</summary>
        Icon = 0x000000100,
        /// <summary>get display name</summary>
        DisplayName = 0x000000200,
        /// <summary>get type name</summary>
        TypeName = 0x000000400,
        /// <summary>get attributes</summary>
        Attributes = 0x000000800,
        /// <summary>get icon location</summary>
        IconLocation = 0x000001000,
        /// <summary>return exe type</summary>
        ExeType = 0x000002000,
        /// <summary>get system icon index</summary>
        SysIconIndex = 0x000004000,
        /// <summary>put a link overlay on icon</summary>
        LinkOverlay = 0x000008000,
        /// <summary>show icon in selected state</summary>
        Selected = 0x000010000,
        /// <summary>get only specified attributes</summary>
        Attr_Specified = 0x000020000,
        /// <summary>get large icon</summary>
        LargeIcon = 0x000000000,
        /// <summary>get small icon</summary>
        SmallIcon = 0x000000001,
        /// <summary>get open icon</summary>
        OpenIcon = 0x000000002,
        /// <summary>get shell size icon</summary>
        ShellIconSize = 0x000000004,
        /// <summary>pszPath is a pidl</summary>
        PIDL = 0x000000008,
        /// <summary>use passed dwFileAttribute</summary>
        UseFileAttributes = 0x000000010,
        /// <summary>apply the appropriate overlays</summary>
        AddOverlays = 0x000000020,
        /// <summary>Get the index of the overlay in the upper 8 bits of the iIcon</summary>
        OverlayIndex = 0x000000040,
    }

    #endregion

    /// <summary>
    /// 
    /// </summary>
    public class Win32Ex
    {
        // This structure will contain information about the file
        public struct SHFILEINFO
        {
            // Handle to the icon representing the file
            public IntPtr hIcon;
            // Index of the icon within the image list
            public int iIcon;
            // Various attributes of the file
            public uint dwAttributes;
            // Path to the file
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
            public string szDisplayName;
            // File type
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
            public string szTypeName;
        };

        // The signature of SHGetFileInfo (located in Shell32.dll)
        [DllImport("Shell32.dll")]
        public static extern IntPtr SHGetFileInfo(string pszPath, 
            uint dwFileAttributes, 
            ref SHFILEINFO psfi, 
            int cbFileInfo, 
            uint uFlags);

        /// <summary>
        /// Get an icon for the file specified
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        public static Icon GetIcon(string fileName, SHGFI flags)
        {
            if (File.Exists(fileName))
            {
                // Get the icon for the file
                SHFILEINFO shinfo = new SHFILEINFO();
                IntPtr iconPtr = SHGetFileInfo(fileName,
                    0,
                    ref shinfo,
                    Marshal.SizeOf(shinfo),
                    (uint)flags);

                // Load the icon from the pointer
                Icon ico = System.Drawing.Icon.FromHandle(shinfo.hIcon);

                // Return the icon
                return ico;
            }
            
            // Return null
            return null;
        }

        /// <summary>
        /// Get the description of the specified file extension
        /// </summary>
        /// <param name="ext"></param>
        /// <returns></returns>
        public static string GetExtenstionDescription(string ext)
        {
            // Open the key 
            RegistryKey rootKey = Registry.ClassesRoot.OpenSubKey(ext);

            // Get the default value
            object defaultValue = rootKey.GetValue("", null);

            if (defaultValue == null)
            {
                return null;
            }
            else
            {
                // Open the sub key
                RegistryKey subKey = Registry.ClassesRoot.OpenSubKey(defaultValue.ToString());

                // Get the default value
                object subKeyDefault = subKey.GetValue("", null);

                if (subKeyDefault == null)
                {
                    return null;
                }

                // Return the default value
                return subKeyDefault.ToString();
            }
        }

    }

}
